@@ -9,6 +9,7 @@ import warnings
 from altgraph import Graph
 import textwrap
 import xml.etree.ElementTree as ET
+import pickle
 
 try:
     bytes
@@ -39,6 +40,39 @@ except AttributeError:
             else:
                 self.fail("unexpected pass")
 
+class TestDependencyInfo (unittest.TestCase):
+    def test_pickling(self):
+        info = modulegraph.DependencyInfo(function=True, conditional=False, tryexcept=True, fromlist=False)
+        for proto in range(pickle.HIGHEST_PROTOCOL + 1):
+            b = pickle.dumps(info, proto)
+            self.assertTrue(isinstance(b, bytes))
+
+            o = pickle.loads(b)
+            self.assertEqual(o, info)
+
+    def test_merging(self):
+        info1 = modulegraph.DependencyInfo(function=True, conditional=False, tryexcept=True, fromlist=False)
+        info2 = modulegraph.DependencyInfo(function=False, conditional=True, tryexcept=True, fromlist=False)
+        self.assertEqual(
+            info1._merged(info2), modulegraph.DependencyInfo(function=True, conditional=True, tryexcept=True, fromlist=False))
+
+        info2 = modulegraph.DependencyInfo(function=False, conditional=True, tryexcept=False, fromlist=False)
+        self.assertEqual(
+            info1._merged(info2), modulegraph.DependencyInfo(function=True, conditional=True, tryexcept=True, fromlist=False))
+
+        info2 = modulegraph.DependencyInfo(function=False, conditional=False, tryexcept=False, fromlist=False)
+        self.assertEqual(
+            info1._merged(info2), modulegraph.DependencyInfo(function=False, conditional=False, tryexcept=False, fromlist=False))
+
+        info1 = modulegraph.DependencyInfo(function=True, conditional=False, tryexcept=True, fromlist=True)
+        self.assertEqual(
+            info1._merged(info2), modulegraph.DependencyInfo(function=False, conditional=False, tryexcept=False, fromlist=False))
+
+        info2 = modulegraph.DependencyInfo(function=False, conditional=False, tryexcept=False, fromlist=True)
+        self.assertEqual(
+            info1._merged(info2), modulegraph.DependencyInfo(function=False, conditional=False, tryexcept=False, fromlist=True))
+
+
 class TestFunctions (unittest.TestCase):
     if not hasattr(unittest.TestCase, 'assertIsInstance'):
         def assertIsInstance(self, obj, types):
@@ -84,28 +118,35 @@ class TestFunctions (unittest.TestCase):
                 raise ValueError("invalid lookup key")
 
         class WS (object):
+            def __init__(self, path=None):
+                pass
+
             def __iter__(self):
                 yield DS("/pkg/pkg1")
                 yield DS("/pkg/pkg2", "foo\n")
                 yield DS("/pkg/pkg3", "bar.baz\n")
                 yield DS("/pkg/pkg4", "foobar\nfoo\n")
 
-        saved_ws = pkg_resources.working_set
+        saved_ws = pkg_resources.WorkingSet
         try:
-            pkg_resources.working_set = WS()
+            pkg_resources.WorkingSet = WS
 
             self.assertEqual(modulegraph._namespace_package_path("sys", ["appdir/pkg"]), ["appdir/pkg"])
-            self.assertEqual(modulegraph._namespace_package_path("foo", ["appdir/pkg"]), ["appdir/pkg", "/pkg/pkg2/foo", "/pkg/pkg4/foo"])
-            self.assertEqual(modulegraph._namespace_package_path("bar.baz", ["appdir/pkg"]), ["appdir/pkg", "/pkg/pkg3/bar/baz"])
+            self.assertEqual(modulegraph._namespace_package_path("foo", ["appdir/pkg"]), ["appdir/pkg", os.path.join("/pkg/pkg2", "foo"), os.path.join("/pkg/pkg4", "foo")])
+            self.assertEqual(modulegraph._namespace_package_path("bar.baz", ["appdir/pkg"]), ["appdir/pkg", os.path.join("/pkg/pkg3", "bar", "baz")])
 
         finally:
-            pkg_resources.working_set = saved_ws
+            pkg_resources.WorkingSet = saved_ws
 
     def test_os_listdir(self):
         root = os.path.join(
                 os.path.dirname(os.path.abspath(__file__)), 'testdata')
 
-        self.assertEqual(modulegraph.os_listdir('/etc/'), os.listdir('/etc'))
+        if os.path.exists('/etc'):
+            self.assertEqual(modulegraph.os_listdir('/etc/'), os.listdir('/etc'))
+        else:
+            self.assertEqual(modulegraph.os_listdir('c:\\'), os.listdir('c:\\'))
+
         self.assertRaises(IOError, modulegraph.os_listdir, '/etc/hosts/foobar')
         self.assertRaises(IOError, modulegraph.os_listdir, os.path.join(root, 'test.egg', 'bar'))
 
@@ -141,9 +182,14 @@ class TestFunctions (unittest.TestCase):
             self.assertTrue(hasattr(fp, 'read'))
 
             if path.endswith('.zip') or path.endswith('.egg'):
-                # Zip importers will precompile
-                self.assertEqual(filename, os.path.join(path, 'mymodule.pyc'))
-                self.assertEqual(description, ('.pyc', 'rb', imp.PY_COMPILED))
+                # Zip importers may precompile
+                if filename.endswith('.py'):
+                    self.assertEqual(filename, os.path.join(path, 'mymodule.py'))
+                    self.assertEqual(description, ('.py', 'rU', imp.PY_SOURCE))
+
+                else:
+                    self.assertEqual(filename, os.path.join(path, 'mymodule.pyc'))
+                    self.assertEqual(description, ('.pyc', 'rb', imp.PY_COMPILED))
 
             else:
                 self.assertEqual(filename, os.path.join(path, 'mymodule.py'))
@@ -335,7 +381,7 @@ class TestNode (unittest.TestCase):
         self.assertEqual(n.get('foo', 'a'), 'a')
         n['foo'] = 42
         self.assertEqual(n['foo'], 42)
-        self.assertTrue('foo' in n)
+        self.assertIn('foo', n)
         self.assertEqual(n._namespace, {'foo':42})
 
     def testOrder(self):
@@ -376,6 +422,9 @@ class TestNode (unittest.TestCase):
         if '__qualname__' in d:
             # New in Python 3.3
             del d['__qualname__']
+        if '__dict__' in d:
+            # New in Python 3.4
+            del d['__dict__']
         self.assertEqual(d, {})
 
     def assertHasExactMethods(self, klass, *methods):
@@ -385,6 +434,9 @@ class TestNode (unittest.TestCase):
         if '__qualname__' in d:
             # New in Python 3.3
             del d['__qualname__']
+        if '__dict__' in d:
+            # New in Python 3.4
+            del d['__dict__']
 
         for nm in methods:
             self.assertTrue(nm in d, "%s doesn't have attribute %r"%(klass, nm))
@@ -583,7 +635,7 @@ class TestModuleGraph (unittest.TestCase):
             self.fail("No package located, should have at least 'os'")
 
         self.assertIsInstance(node, modulegraph.Package)
-        parent = graph.determine_parent(node)
+        parent = graph._determine_parent(node)
         self.assertEqual(parent.identifier, node.identifier)
         self.assertEqual(parent, graph.findNode(node.identifier))
         self.assertTrue(isinstance(parent, modulegraph.Package))
@@ -591,14 +643,14 @@ class TestModuleGraph (unittest.TestCase):
         # XXX: Might be a usecase for some odd code in determine_parent...
         #node = modulegraph.Package('encodings')
         #node.packagepath = parent.packagepath
-        #m = graph.determine_parent(node)
+        #m = graph._determine_parent(node)
         #self.assertTrue(m is parent)
 
         m = graph.findNode('xml')
-        self.assertEqual(graph.determine_parent(m), m)
+        self.assertEqual(graph._determine_parent(m), m)
 
         m = graph.findNode('xml.dom')
-        self.assertEqual(graph.determine_parent(m), graph.findNode('xml.dom'))
+        self.assertEqual(graph._determine_parent(m), graph.findNode('xml.dom'))
 
 
     @expectedFailure
@@ -606,27 +658,28 @@ class TestModuleGraph (unittest.TestCase):
         self.fail("find_head_package")
 
     def test_load_tail(self):
+        # XXX: This test is dodgy!
         graph = modulegraph.ModuleGraph()
 
         record = []
-        def import_module(partname, fqname, parent):
+        def _import_module(partname, fqname, parent):
             record.append((partname, fqname, parent))
-            if partname == 'raises':
+            if partname == 'raises' or '.raises.' in fqname:
                 return None
             return modulegraph.Node(fqname)
 
-        graph.import_module = import_module
+        graph._import_module = _import_module
 
         record = []
         root = modulegraph.Node('root')
-        m = graph.load_tail(root, '')
+        m = graph._load_tail(root, '')
         self.assertTrue(m is root)
         self.assertEqual(record, [
             ])
 
         record = []
         root = modulegraph.Node('root')
-        m = graph.load_tail(root, 'sub')
+        m = graph._load_tail(root, 'sub')
         self.assertFalse(m is root)
         self.assertEqual(record, [
                 ('sub', 'root.sub', root),
@@ -634,7 +687,7 @@ class TestModuleGraph (unittest.TestCase):
 
         record = []
         root = modulegraph.Node('root')
-        m = graph.load_tail(root, 'sub.sub1')
+        m = graph._load_tail(root, 'sub.sub1')
         self.assertFalse(m is root)
         node = modulegraph.Node('root.sub')
         self.assertEqual(record, [
@@ -644,7 +697,7 @@ class TestModuleGraph (unittest.TestCase):
 
         record = []
         root = modulegraph.Node('root')
-        m = graph.load_tail(root, 'sub.sub1.sub2')
+        m = graph._load_tail(root, 'sub.sub1.sub2')
         self.assertFalse(m is root)
         node = modulegraph.Node('root.sub')
         node2 = modulegraph.Node('root.sub.sub1')
@@ -654,9 +707,17 @@ class TestModuleGraph (unittest.TestCase):
                 ('sub2', 'root.sub.sub1.sub2', node2),
             ])
 
-        self.assertRaises(ImportError, graph.load_tail, root, 'raises')
-        self.assertRaises(ImportError, graph.load_tail, root, 'sub.raises')
-        self.assertRaises(ImportError, graph.load_tail, root, 'sub.raises.sub')
+        n = graph._load_tail(root, 'raises')
+        self.assertIsInstance(n, modulegraph.MissingModule)
+        self.assertEqual(n.identifier, 'root.raises')
+
+        n = graph._load_tail(root, 'sub.raises')
+        self.assertIsInstance(n, modulegraph.MissingModule)
+        self.assertEqual(n.identifier, 'root.sub.raises')
+
+        n = graph._load_tail(root, 'sub.raises.sub')
+        self.assertIsInstance(n, modulegraph.MissingModule)
+        self.assertEqual(n.identifier, 'root.sub.raises.sub')
 
 
 
@@ -760,7 +821,7 @@ class TestModuleGraph (unittest.TestCase):
             modulegraph.find_module = mock_finder
 
             graph = modulegraph.ModuleGraph()
-            m = graph.find_module('sys', None)
+            m = graph._find_module('sys', None)
             self.assertEqual(record, [])
             self.assertEqual(m, (None, None, ("", "", imp.C_BUILTIN)))
 
@@ -769,10 +830,10 @@ class TestModuleGraph (unittest.TestCase):
             self.assertEqual(xml.identifier, 'xml')
             modulegraph.find_module = mock_finder
 
-            self.assertRaises(ImportError, graph.find_module, 'xml', None)
+            self.assertRaises(ImportError, graph._find_module, 'xml', None)
 
             self.assertEqual(record, [])
-            m = graph.find_module('shutil', None)
+            m = graph._find_module('shutil', None)
             self.assertEqual(record, [
                 ('shutil', graph.path),
             ])
@@ -783,17 +844,17 @@ class TestModuleGraph (unittest.TestCase):
             srcfn = shutil.__file__
             if srcfn.endswith('.pyc'):
                 srcfn = srcfn[:-1]
-            self.assertEqual(m[1], srcfn)
+            self.assertEqual(os.path.realpath(m[1]), os.path.realpath(srcfn))
             self.assertEqual(m[2], ('.py', 'rU', imp.PY_SOURCE))
             m[0].close()
 
-            m2 = graph.find_module('shutil', None)
+            m2 = graph._find_module('shutil', None)
             self.assertEqual(m[1:], m2[1:])
             m2[0].close()
 
 
             record[:] = []
-            m = graph.find_module('sax', xml.packagepath, xml)
+            m = graph._find_module('sax', xml.packagepath, xml)
             self.assertEqual(m,
                     (None, os.path.join(os.path.dirname(xml.filename), 'sax'),
                     ('', '', imp.PKG_DIRECTORY)))
@@ -886,43 +947,43 @@ class TestModuleGraph (unittest.TestCase):
     def test_replace_paths_in_code(self):
         graph = modulegraph.ModuleGraph(replace_paths=[
                 ('path1', 'path2'),
-                ('path3/path5', 'path4'),
+                (os.path.join('path3', 'path5'), 'path4'),
             ])
 
         co = compile(textwrap.dedent("""
         [x for x in range(4)]
-        """), "path4/index.py", 'exec', 0, 1)
-        co = graph.replace_paths_in_code(co)
-        self.assertEqual(co.co_filename, 'path4/index.py')
+        """), os.path.join("path4", "index.py"), 'exec', 0, 1)
+        co = graph._replace_paths_in_code(co)
+        self.assertEqual(co.co_filename, os.path.join('path4', 'index.py'))
 
         co = compile(textwrap.dedent("""
         [x for x in range(4)]
         (x for x in range(4))
-        """), "path1/index.py", 'exec', 0, 1)
-        self.assertEqual(co.co_filename, 'path1/index.py')
-        co = graph.replace_paths_in_code(co)
-        self.assertEqual(co.co_filename, 'path2/index.py')
+        """), os.path.join("path1", "index.py"), 'exec', 0, 1)
+        self.assertEqual(co.co_filename, os.path.join('path1', 'index.py'))
+        co = graph._replace_paths_in_code(co)
+        self.assertEqual(co.co_filename, os.path.join('path2', 'index.py'))
         for c in co.co_consts:
             if isinstance(c, type(co)):
-                self.assertEqual(c.co_filename, 'path2/index.py')
+                self.assertEqual(c.co_filename, os.path.join('path2', 'index.py'))
 
         co = compile(textwrap.dedent("""
         [x for x in range(4)]
-        """), "path3/path4/index.py", 'exec', 0, 1)
-        co = graph.replace_paths_in_code(co)
-        self.assertEqual(co.co_filename, 'path3/path4/index.py')
+        """), os.path.join("path3", "path4", "index.py"), 'exec', 0, 1)
+        co = graph._replace_paths_in_code(co)
+        self.assertEqual(co.co_filename, os.path.join('path3', 'path4', 'index.py'))
 
         co = compile(textwrap.dedent("""
         [x for x in range(4)]
-        """), "path3/path5.py", 'exec', 0, 1)
-        co = graph.replace_paths_in_code(co)
-        self.assertEqual(co.co_filename, 'path3/path5.py')
+        """), os.path.join("path3", "path5.py"), 'exec', 0, 1)
+        co = graph._replace_paths_in_code(co)
+        self.assertEqual(co.co_filename, os.path.join('path3', 'path5.py'))
 
         co = compile(textwrap.dedent("""
         [x for x in range(4)]
-        """), "path3/path5/index.py", 'exec', 0, 1)
-        co = graph.replace_paths_in_code(co)
-        self.assertEqual(co.co_filename, 'path4/index.py')
+        """), os.path.join("path3", "path5", "index.py"), 'exec', 0, 1)
+        co = graph._replace_paths_in_code(co)
+        self.assertEqual(co.co_filename, os.path.join('path4', 'index.py'))
 
     def test_createReference(self):
         graph = modulegraph.ModuleGraph()
@@ -977,7 +1038,9 @@ class TestModuleGraph (unittest.TestCase):
         graph.import_hook('os')
         graph.import_hook('xml.etree')
         graph.import_hook('unittest')
-        graph.import_hook('distutils.command.build')
+        # Don't use "distutils.command.build" here, causes problems on
+        # Travis.
+        graph.import_hook('lib2to3.fixes.fix_apply')
 
         fp = StringIO()
         list(graph.itergraphreport())
